import functools
import ctypes
import sys


def is_admin():
    return ctypes.windll.shell32.IsUserAnAdmin()


class _Admin:
    def __init__(self, func):
        self.func = func
        functools.update_wrapper(self, func)

    def __call__(self, *args, **kwargs):
        if is_admin():
            self.func(*args, **kwargs)
        else:
            # Re-run the program with admin rights
            ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, " ".join(sys.argv), None, 1)
            self.func(*args, **kwargs)


def admin(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return _Admin(func)(*args, **kwargs)
    return wrapper



import functools
import datetime
import inspect


class _Debug:
    register = dict()

    def __init__(self, func):
        self.func = func
        self.register[func.__name__] = self

    def __call__(self, *args, **kwargs):
        print(f'\x1b[38;5;117m'
              f'[{datetime.datetime.now().strftime("%H:%M:%S")}]'
              f'\x1b[0;m', end=' ')
        print(f'\x1b[38;5;117m'
              f'"{self.func.__name__}" was called:\n'
              f'Caller: {inspect.stack()[2].function}\n'
              f'Params: {args, kwargs}'
              f'\x1b[0;m')

        print(f'\x1b[38;5;117m'
              f'Loc: line {inspect.stack()[2].lineno} in "{inspect.stack()[2].filename}"'
              f'\x1b[0;m')

        result = self.func(*args, **kwargs)

        print(f'\x1b[38;5;117m'
              f'[{datetime.datetime.now().strftime("%H:%M:%S")}]'
              f'\x1b[0;m', end=' ')
        print(f'\x1b[38;5;117m'
              f'"{self.func.__name__}" returned with: {result}'
              f'\n'
              f'\x1b[0;m')

        return result


def debug(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return _Debug(func)(*args, **kwargs)
    return wrapper


def d_print(*args, **kwargs):
    caller = inspect.stack()[1].function
    if caller in _Debug.register.keys():
        print(f'\x1b[38;5;117m'
              f'[{datetime.datetime.now().strftime("%H:%M:%S")}]'
              f'\x1b[0;m', end=' ')
        print(f'\x1b[38;5;117m', end='')
        print(*args, **kwargs, end='')
        print(f'\x1b[0;m')



import functools


class _Cache:
    register = dict()

    def __init__(self, func):
        self.func = func

    def __call__(self, *args, **kwargs):
        arg_key = f'{self.func.__name__}-{args}-{kwargs}'
        if arg_key in self.register.keys():
            return self.register[arg_key]
        else:
            result = self.func(*args, **kwargs)
            self.register[arg_key] = result
            return result


def cache(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return _Cache(func)(*args, **kwargs)
    return wrapper



import os


def clean(path):
    cache = list()
    with open(path, 'r') as f:
        for i in f.readlines():
            if '@debug' in i or 'd_print' in i:
                continue
            else:
                cache.append(i)
    result = ''.join(cache)
    directory, file_name = os.path.split(path)
    new_name = f'cleaned_{file_name}'
    new_path = f'{directory}\\{new_name}'
    with open(new_path, 'w') as f:
        f.write(result)


if __name__ == '__main__':
    clean('.\\cache.py')
